#pragma once
#include <kr2_program_api/api_v1/program_interface.h>
#include <kr2_program_api/api_v1/bundles/bundle_class.h>
#include <domain/physical_quantities.h>
#include <domain/program.h>
#include <communication/bridge/proxy/i_tool_controller.h>
#include <chrono>

namespace spin
{
    class RobotMotion
    {
    public:
        RobotMotion(kr2_program_api::ProgramInterface *api, std::shared_ptr<IToolController> toolController);
        void moveTo(const kr2_program_api::RobotPose &referencePose, const Speed &speed, const Length &zOffset);
        void moveTo(const kr2_program_api::RobotPose &pose, const Speed &speed);
        void wait(const std::chrono::nanoseconds &duration);
        void moveAsyncTo(const kr2_program_api::RobotPose &screwPose, const Speed &speed, const Length &zOffset = Length::fromMeters(0.0));
        void safeApproach(const Program &program, const kr2_program_api::RobotPose &screwPose);
        void safeDepart(const Program &program, const kr2_program_api::RobotPose &screwPose);
        void showInfoDialog(const std::string &message);
        void showFaultDialog(const Program &Program);
        void showOKDialog(const Program &Program);
        void showNotOKDialog(const Program &Program);
        void onProgramTerminated(const kr2_signal::ProgramTerminated &event);

    private:
        kr2_program_api::ProgramInterface *api_;
        std::shared_ptr<IToolController> toolController_;
        kr2_program_api_internals::BusSubscriber subscriber_;
        std::atomic<bool> isProgramTerminated_{false};
    };

} // namespace spin